@@ -58,7 +58,7 @@ import java.util.List;
 public final class DexFileFactory {
 
     @Nonnull
-    public static DexBackedDexFile loadDexFile(@Nonnull String path, @Nonnull Opcodes opcodes) throws IOException {
+    public static DexBackedDexFile loadDexFile(@Nonnull String path, @Nullable Opcodes opcodes) throws IOException {
         return loadDexFile(new File(path), opcodes);
     }
 
@@ -79,7 +79,7 @@ public final class DexFileFactory {
      * in a zip file is not a valid dex file
      */
     @Nonnull
-    public static DexBackedDexFile loadDexFile(@Nonnull File file, @Nonnull Opcodes opcodes) throws IOException {
+    public static DexBackedDexFile loadDexFile(@Nonnull File file, @Nullable Opcodes opcodes) throws IOException {
         if (!file.exists()) {
             throw new DexFileNotFoundException("%s does not exist", file.getName());
         }
@@ -176,7 +176,7 @@ public final class DexFileFactory {
      * @throws MultipleMatchingDexEntriesException If multiple entries match the given dexEntry
      */
     public static DexBackedDexFile loadDexEntry(@Nonnull File file, @Nonnull String dexEntry,
-                                                boolean exactMatch, @Nonnull Opcodes opcodes) throws IOException {
+                                                boolean exactMatch, @Nullable Opcodes opcodes) throws IOException {
         if (!file.exists()) {
             throw new DexFileNotFoundException("Container file %s does not exist", file.getName());
         }
@@ -230,7 +230,7 @@ public final class DexFileFactory {
      * @throws UnsupportedFileTypeException If the given file is not a valid dex/zip/odex/oat file
      */
     public static MultiDexContainer<? extends DexBackedDexFile> loadDexContainer(
-            @Nonnull File file, @Nonnull final Opcodes opcodes) throws IOException {
+            @Nonnull File file, @Nullable final Opcodes opcodes) throws IOException {
         if (!file.exists()) {
             throw new DexFileNotFoundException("%s does not exist", file.getName());
         }
@@ -456,10 +456,6 @@ public final class DexFileFactory {
             }
             return null;
         }
-
-        @Nonnull @Override public Opcodes getOpcodes() {
-            return dexFile.getOpcodes();
-        }
     }
 
     public static class FilenameVdexProvider implements VdexProvider {
